﻿/*
*	りふと
*/

class LiftParam extends CSqParam
{
	stopFrame		= 0
	maxSpeedFrame	= 0
	angle			= 0.0
	defPos			= SPoint2DF()
}

class Lift extends GameObjectBase
{
	/*
	*	コンストラクタ
	*/
	constructor()
	{
		// テクスチャ登録
		this.RegistTexture( "mapchi_obj.png", "Lift", 50000, 120, 0, 80, 16 )
		
		// 各パラメータ設定
		local obj = GameObject.IObject()
		obj.Texture	= "Lift"
		
		// コリジョン設定
		local detail = GameObject.SColDetail.ctorRect( -40, -8, 40, 8 )
		detail.Flag = (ColFlagD.TreatMap | ColFlagD.ColMap)
		obj.Col.PushDetail( detail )
		
		// 登録
		GameObjectBase.constructor( obj, "Lift", eObjType.Object )
	}
	
	
	/*
	*	初期化
	*/
	function Init(o)
	{
		o.SqParam	= LiftParam()
		local fP = o.FreeParam
		local prm = o.SqParam
		
		if( fP.L0 != 0 ){
			prm.angle	= fP.F0
			prm.defPos	= o.Pos.Clone()
		}
	}
	
	
	/*
	*	通常動作
	*/
	function Move(o)
	{
		local fP = o.FreeParam
		local prm = o.SqParam
		prm.Update()
		
		local oldPos = o.Pos.Clone()
		
		if( fP.L0 == 0 ){
			/*
			*	F0 : 最高速度
			*	F1 : 加速度
			*/
			if( --prm.stopFrame <= 0 ){
				local oldSpeed = o.Speed.Clone()
				
				if( o.Dir & (DIR_UP | DIR_DOWN) ){
					if( fP.F1 == 0.0 ){
						o.Speed.Y = fP.F0 * (o.Dir & DIR_UP ? -1.0 : 1.0)
						if( ++prm.maxSpeedFrame >= fP.L1 ){ prm.maxSpeedFrame = 0; prm.stopFrame = fP.L2; o.Speed.Y = 0; o.Dir = o.Dir ^ (DIR_UP | DIR_DOWN) }
					}
					else{
						if( ++prm.maxSpeedFrame >= fP.L1 ){
							o.Speed.Y += fP.F1 * (o.Dir & DIR_UP ? -1.0 : 1.0)
							
							if( o.Dir & DIR_UP ){ if( o.Speed.Y < -fP.F0 ){ prm.maxSpeedFrame = 0; o.Speed.Y = -fP.F0; o.Dir = o.Dir ^ (DIR_UP | DIR_DOWN) } }
							else                { if( o.Speed.Y >  fP.F0 ){ prm.maxSpeedFrame = 0; o.Speed.Y =  fP.F0; o.Dir = o.Dir ^ (DIR_UP | DIR_DOWN) } }
							
							if( ((oldSpeed.Y < 0) && (o.Speed.Y >= 0)) || ((oldSpeed.Y > 0) && (o.Speed.Y <= 0)) ){
								o.Speed.Y = 0
								prm.stopFrame = fP.L2
							}
						}
					}
				}
				else if( o.Dir & (DIR_LEFT | DIR_RIGHT) ){
					if( fP.F1 == 0.0 ){
						o.Speed.X = fP.F0 * (o.Dir & DIR_LEFT ? -1.0 : 1.0)
						if( ++prm.maxSpeedFrame >= fP.L1 ){ prm.maxSpeedFrame = 0; prm.stopFrame = fP.L2; o.Speed.X = 0; o.Dir = o.Dir ^ (DIR_LEFT | DIR_RIGHT) }
					}
					else{
						if( ++prm.maxSpeedFrame >= fP.L1 ){
							o.Speed.X += fP.F1 * (o.Dir & DIR_LEFT ? -1.0 : 1.0)
							
							if( o.Dir & DIR_LEFT ){ if( o.Speed.X < -fP.F0 ){ prm.maxSpeedFrame = 0; o.Speed.X = -fP.F0; o.Dir = o.Dir ^ (DIR_LEFT | DIR_RIGHT) } }
							else                  { if( o.Speed.X >  fP.F0 ){ prm.maxSpeedFrame = 0; o.Speed.X =  fP.F0; o.Dir = o.Dir ^ (DIR_LEFT | DIR_RIGHT) } }
							
							if( ((oldSpeed.X < 0) && (o.Speed.X >= 0)) || ((oldSpeed.X > 0) && (o.Speed.X <= 0)) ){
								o.Speed.X = 0
								prm.stopFrame = fP.L2
							}
						}
					}
				}
			}
		}
		else{
			/*
			*	L0 : (L0 < 0) --> 逆時計、(L0 > 0) --> 時計
			*
			*	F0 : 初期角度
			*	F1 : 半径
			*	F2 : 角速度
			*/
			prm.angle += fP.F2 * (fP.L0 < 0 ? -1 : 1)
			local rad = prm.angle * PI * 2
			
			o.Pos = prm.defPos + SPoint2DF.ctor( fP.F1 * cos( rad ), fP.F1 * sin( rad ) )
			o.Speed = o.Pos - oldPos
		}
		
		
		
		// 載ってるオブジェクトを動かす
		if( prm.landObjList.len() ){
			foreach(v in prm.landObjList){ v.Pos += o.Speed; if( o.Speed.Y > 0 ) v.Pos.Y += o.Speed.Y }
			prm.landObjList.clear()
		}
		
		
		if( fP.L0 != 0 ){
			o.Speed.Set(0)
		}
	}
	
	
	function ColTreatMap(o1, o2, dir)
	{
		local prm1 = o1.SqParam
		local prm2 = o2.SqParam
		
		if( dir & DIR_DOWN ) prm1.landObjList.push( o2 )
	}
}


Lift()
