﻿/*
*	気箱
*/

class SlimeParam extends CSqParam
{
	speed				= null
	notFall				= null
	notFallTurn			= null
	boundPow			= null
	beforeStopSpeed		= null
	beforeStopSpeedFlag	= null
	
	constructor(boundPowX, boundPowY)
	{
		CSqParam.constructor()
		
		speed				= { walk = 0, fall = 0 }
		notFall				= true
		notFallTurn			= 0
		boundPow			= { x = boundPowX, y = boundPowY }
		beforeStopSpeed		= SPoint2DF()
		beforeStopSpeedFlag	= false
	}
}


class SlimeBase extends GameObjectBase
{
	/*
	*	コンストラクタ
	*/
	constructor(name, srcPointY)
	{
		// テクスチャ登録
		this.RegistTexture( "poyoyon.png", name, 50002, 0, 40 * srcPointY * 3, 40, 40 )
		
		// 各パラメータ設定
		local obj = GameObject.IObject()
		obj.Texture	= name
		
		// 登録
		GameObjectBase.constructor( obj, name, eObjType.Object )
	}
	
	
	/*
	*	初期化
	*/
	function Init(o, boundPowX, boundPowY)
	{
		o.SqParam	= SlimeParam( boundPowX, boundPowY )
		local prm = o.SqParam
		local fP = o.FreeParam
		
		
		// 向きを強制
		if     ( o.Dir & DIR_DOWN  ) o.Dir = DIR_DOWN
		else if( o.Dir & DIR_LEFT  ) o.Dir = DIR_LEFT
		else if( o.Dir & DIR_RIGHT ) o.Dir = DIR_RIGHT
		else                         o.Dir = DIR_UP
		
		
		// あんぐる設定
		switch( o.Dir ){
			case DIR_UP:	o.Angle = 0.75;	break
			case DIR_DOWN:	o.Angle = 0.25;	break
			case DIR_LEFT:	o.Angle = 0.5;	break
			case DIR_RIGHT:	o.Angle = 0.0;	break
		}
		
		
		// フリパラから速度関係
		/*
		*	L0: 落ちないふらぐ
		*
		*	F0: x
		*	F1: y
		*/
		
		if( fP.F0 != 0 ){
			prm.speed.walk = fP.F0
			if( o.Dir & (DIR_UP | DIR_DOWN) ) o.Speed.X = fP.F0
			else                              o.Speed.Y = fP.F0
		}

		if     ( fP.F1 <  0 ) prm.speed.fall = 0.0
		else if( fP.F1 == 0 ) prm.speed.fall = 0.4
		else                  prm.speed.fall = fP.F1
		
		prm.notFall = (fP.L0 == 0)
		
		
		// コリジョン関係
		local rect = SRectF()	
		
		switch( o.Dir ){
			case DIR_UP:	rect.Set( -16,   0, 16, 20 );	break
			case DIR_DOWN:	rect.Set( -16, -20, 16,  0 );	break
			case DIR_LEFT:	rect.Set(   0, -16, 20, 16 );	break
			case DIR_RIGHT:	rect.Set( -20, -16,  0, 16 );	break
		}
		if( prm.boundPow.x == 0 ){
			switch( o.Dir ){
				case DIR_UP:	rect.Sy += 4;	break
				case DIR_DOWN:	rect.Ey -= 4;	break
				case DIR_LEFT:	rect.Sx += 4;	break
				case DIR_RIGHT:	rect.Ex -= 4;	break
			}
		}

		local detail = GameObject.SColDetail.ctorRect( rect.Sx, rect.Sy, rect.Ex, rect.Ey )
		detail.Flag = ColFlagD.TreatMap
		if( (prm.speed.walk != 0) || (prm.speed.fall != 0) ) detail.Flag = detail.Flag | ColFlagD.ColMap
		o.Col.PushDetail( detail )
	}
	
	
	/*
	*	通常動作
	*/
	function Move(o)
	{
		local prm = o.SqParam
		prm.Update()
		
		switch( o.Dir ){
			case DIR_UP:
				local a = (prm.boundDir == DIR_INVALID) && (o.ColDir & (DIR_LEFT | DIR_RIGHT))
				local b = prm.notFall && (prm.notFallTurn <= 0) && (o.ColDir & DIR_DOWN) && !(o.ColMat & 0x02) && (!(o.ColMat & 0x01) || !(o.ColMat & 0x04))
				
				if( a || b ){ o.Speed.X *= -1; prm.notFallTurn = 2 }
				break
				
			case DIR_DOWN:
				local a = (prm.boundDir == DIR_INVALID) && (o.ColDir & (DIR_LEFT | DIR_RIGHT))
				local b = prm.notFall && (prm.notFallTurn <= 0) && (o.ColDir & DIR_UP) && !(o.ColMat & 0x40) && (!(o.ColMat & 0x20) || !(o.ColMat & 0x80))
				
				if( a || b ){ o.Speed.X *= -1; prm.notFallTurn = 2 }
				break
				
			case DIR_LEFT:
				local a = (prm.boundDir == DIR_INVALID) && (o.ColDir & (DIR_UP | DIR_DOWN))
				local b = prm.notFall && (prm.notFallTurn <= 0) && (o.ColDir & DIR_RIGHT) && !(o.ColMat & 0x10) && (!(o.ColMat & 0x04) || !(o.ColMat & 0x80))
				
				if( a || b ){ o.Speed.Y *= -1; prm.notFallTurn = 2 }
				break
				
			case DIR_RIGHT:
				local a = (prm.boundDir == DIR_INVALID) && (o.ColDir & (DIR_UP | DIR_DOWN))
				local b = prm.notFall && (prm.notFallTurn <= 0) && (o.ColDir & DIR_LEFT) && !(o.ColMat & 0x08) && (!(o.ColMat & 0x01) || !(o.ColMat & 0x20))
				
				if( a || b ){ o.Speed.Y *= -1; prm.notFallTurn = 2 }
				break
		}
		--prm.notFallTurn
		
		
		if( prm.speed.walk == 0 ){
			if( o.Dir & (DIR_UP | DIR_DOWN) ) o.Speed.X = 0
			else                              o.Speed.Y = 0
		}
		else{
			if( ((o.Dir == DIR_UP) && (o.ColDir & DIR_DOWN)) || ((o.Dir == DIR_DOWN) && (o.ColDir & DIR_UP)) ){
				local spd = prm.speed.walk * (prm.speed.walk < 0 ? -1 : 1)
				if     ( o.Speed.X < -spd ){ if( (o.Speed.X *= 0.8) > -spd ) o.Speed.X = -spd }
				else if( o.Speed.X >  spd ){ if( (o.Speed.X *= 0.8) <  spd ) o.Speed.X =  spd }
			}
			else if( ((o.Dir == DIR_LEFT) && (o.ColDir & DIR_RIGHT)) || ((o.Dir == DIR_RIGHT) && (o.ColDir & DIR_LEFT)) ){
				local spd = prm.speed.walk * (prm.speed.walk < 0 ? -1 : 1)
				if     ( o.Speed.Y < -spd ){ if( (o.Speed.Y *= 0.8) > -spd ) o.Speed.Y = -spd }
				else if( o.Speed.Y >  spd ){ if( (o.Speed.Y *= 0.8) <  spd ) o.Speed.Y =  spd }
			}
		}
		
		
		// 落下
		if( prm.speed.fall == 0 ){
			if( o.Dir & (DIR_UP | DIR_DOWN) ) o.Speed.Y = 0
			else                              o.Speed.X = 0
		}
		else{
			switch( o.Dir ){
				case DIR_UP:
				case DIR_DOWN:
					if( o.ColDir & (DIR_UP | DIR_DOWN) ){ if( prm.boundDir == DIR_INVALID ) o.Speed.Y = 0 }
					break
				case DIR_LEFT:
				case DIR_RIGHT:
					if( o.ColDir & (DIR_LEFT | DIR_RIGHT) ){ if( prm.boundDir == DIR_INVALID ) o.Speed.X = 0 }
					break
			}
			switch( o.Dir ){
				case DIR_UP:	o.Speed.Y += prm.speed.fall;	break
				case DIR_DOWN:	o.Speed.Y -= prm.speed.fall;	break
				case DIR_LEFT:	o.Speed.X += prm.speed.fall;	break
				case DIR_RIGHT:	o.Speed.X -= prm.speed.fall;	break
			}
		}
		prm.boundDir = DIR_INVALID
		
		
		switch( prm.state ){
			case 0:
				if( prm.frame >= 6 * 3 ) prm.Reset()
				else                     prm.stateSub = prm.frame / 6
				break
			case 1:
				if( prm.frame >= 4 * 5 ) prm.Reset()
				else                     prm.stateSub = prm.frame / 4
				break
			case 2:
			case 3:
				if( prm.frame >= 1 * 14 ) prm.Reset()
				else                      prm.stateSub = prm.frame / 1
				break
		}
		
		
		if( prm.boundPow.x == 0 ){
			switch( prm.state ){
				case 0:
					if( prm.speed.walk != 0 ){
						switch( o.Dir ){
							case DIR_UP:
							case DIR_DOWN:
								if( prm.beforeStopSpeedFlag ){
									o.Speed = prm.beforeStopSpeed
									prm.beforeStopSpeedFlag = false
								}
								break
								
							case DIR_LEFT:
							case DIR_RIGHT:
								if( prm.beforeStopSpeedFlag ){
									o.Speed = prm.beforeStopSpeed
									prm.beforeStopSpeedFlag = false
								}
								break
						}
					}
					break
				case 1:
					if( !prm.beforeStopSpeedFlag ){
						prm.beforeStopSpeedFlag = true
						prm.beforeStopSpeed = o.Speed.Clone()
					}
					o.Speed.Set(0)
					break
				case 2:
				case 3:
					if( !prm.beforeStopSpeedFlag ){
						prm.beforeStopSpeedFlag = true
						prm.beforeStopSpeed = o.Speed.Clone()
					}
					o.Speed.Set(0)
					break
			}
		}
		
		// 載ってるオブジェクトを動かす
		if( prm.landObjList.len() ){
			foreach(v in prm.landObjList) v.Pos += o.Speed
			prm.landObjList.clear()
		}
	}
	
	
	/*
	*	描画
	*/
	function Draw(o)
	{
		local prm = o.SqParam
		
		local srcPoint = { X = 0, Y = 0 }
		o.ApplyDefSrc()
		
		
		switch( prm.state ){
			case 0:
				srcPoint.Y += [ 0, 1, 2 ][ prm.stateSub ]
				break
			case 1:
				srcPoint.X += [ 0, 1, 2, 0, 3 ][ prm.stateSub ]
				break
			case 2:
				if( prm.stateSub <= 6 ) srcPoint.Y += 1
				else                    srcPoint.Y += 2
				srcPoint.X += [ 0, 1, 2, 3, 2, 1, 0, 0, 1, 2, 3, 2, 1, 0 ][ prm.stateSub ]
				break
			case 3:
				if( prm.stateSub <= 6 ) srcPoint.Y += 2
				else                    srcPoint.Y += 1
				srcPoint.X += [ 0, 1, 2, 3, 2, 1, 0, 0, 1, 2, 3, 2, 1, 0 ][ prm.stateSub ]
				break
		}
		
		o.Src.X += o.Src.W * srcPoint.X
		o.Src.Y += o.Src.H * srcPoint.Y
	}
	
	
	/*
	*	地形あたり
	*/
	function ColTreatMap(o1, o2, dir)
	{
		local prm1 = o1.SqParam
		local prm2 = o2.SqParam
		
		local oldState1 = prm1.stateSub
		

		// 自分
		prm1.Reset()
		if     ( dir & DIR_UP    ) prm1.state = (o1.Dir & (DIR_LEFT | DIR_RIGHT) ? 2 : 1)
		else if( dir & DIR_DOWN  ) prm1.state = (o1.Dir & (DIR_LEFT | DIR_RIGHT) ? 2 : 1)
		else if( dir & DIR_LEFT  ) prm1.state = (o1.Dir & (DIR_LEFT | DIR_RIGHT) ? 1 : (o1.Dir & DIR_DOWN ? 3 : 2))
		else if( dir & DIR_RIGHT ) prm1.state = (o1.Dir & (DIR_LEFT | DIR_RIGHT) ? 1 : (o1.Dir & DIR_DOWN ? 2 : 3))
		
		
		
		// 相手
		if( prm1.boundPow.x != 0 ){
			if( dir & DIR_UP ){
				prm2.boundDir = DIR_UP
				o2.Speed.Y = prm1.boundPow.y
				
				if( prm2 instanceof SlimeParam ){ o1.Speed.Y = -prm2.boundPow.y; prm1.boundDir = DIR_DOWN }
				
				if( o1.Dir & (DIR_LEFT | DIR_RIGHT) ) prm1.state = 2
			}
			else if( dir & DIR_DOWN ){
				prm2.boundDir = DIR_DOWN
				o2.Speed.Y = -prm1.boundPow.y
				
				if( prm2 instanceof SlimeParam ){ o1.Speed.Y = prm2.boundPow.y; prm1.boundDir = DIR_UP }
				
				if( o1.Dir & (DIR_LEFT | DIR_RIGHT) ) prm1.state = 2
			}
			else if( dir & DIR_LEFT ){
				prm2.boundDir = DIR_LEFT
				o2.Speed.X = prm1.boundPow.x
				
				if( prm2 instanceof SlimeParam ){ o1.Speed.X = -prm2.boundPow.y; prm1.boundDir = DIR_RIGHT; o2.Speed.X = prm1.boundPow.y }
				
				if( o1.Dir & (DIR_LEFT | DIR_RIGHT) ) prm1.state = 1
			}
			else if( dir & DIR_RIGHT ){
				prm2.boundDir = DIR_RIGHT
				o2.Speed.X = -prm1.boundPow.x
				
				if( prm2 instanceof SlimeParam ){ o1.Speed.X = prm2.boundPow.y; prm1.boundDir = DIR_LEFT; o2.Speed.X = -prm1.boundPow.y }
				
				if( o1.Dir & (DIR_LEFT | DIR_RIGHT) ) prm1.state = 1
			}
		}
		
		
		if( oldState1 != prm1.state ){
			prm1.frame		= 0
			prm1.stateSub	= 0
		}
		
		if( prm1.boundPow.x == 0 ){
		
			if( dir & DIR_DOWN ) prm1.landObjList.push( o2 )
			
			switch( prm1.state ){
				case 1:
					prm1.frame = 4
					prm1.stateSub = prm1.frame / 4
					break
				case 2:
				case 3:
					prm1.frame = 2
					prm1.stateSub = prm1.frame / 1
					break
			}
		}
		
		
		// 音
		if( prm1.boundPow.x != 0 ){
			if( prm2 instanceof SheepParam ){
				local xau = My.CXAudio()
				local se = xau.Find( (prm1.state != 1 ? "slime04" : "slime05") )
				se.Play()
			}
		}
	}
}




class Slime extends SlimeBase
{
	constructor()		{ SlimeBase.constructor( "Slime", 0 ) }
	function Init(o)	{ SlimeBase.Init( o, 15, 10.5 ) }
}
class SlimeNoBound extends SlimeBase
{
	constructor()		{ SlimeBase.constructor( "SlimeNoBound", 3 ) }
	function Init(o)	{ SlimeBase.Init( o, 0, 0 ) }
}
class SlimePower extends SlimeBase
{
	constructor()		{ SlimeBase.constructor( "SlimePower", 2 ) }
	function Init(o)	{ SlimeBase.Init( o, 25, 14.9 ) }
}
class SlimeHonnori extends SlimeBase
{
	constructor()		{ SlimeBase.constructor( "SlimeHonnori", 1 ) }
	function Init(o)	{ SlimeBase.Init( o, 7.5, 7.5 ) }
}


Slime()
SlimeNoBound()
SlimePower()
SlimeHonnori()
