﻿/*
*	プレイヤー
*/


playerParam <-
{
	oldSpeed	= SPoint2DF()
}

dofile( "./data/squirrel/object/common/player/player_walk.nut" )
dofile( "./data/squirrel/object/common/player/player_jump.nut" )
dofile( "./data/squirrel/object/common/player/player_knockback.nut" )
dofile( "./data/squirrel/object/common/player/player_attack.nut" )


shotValue <- null

maxSpeed	<- { X = 5, Y = 9 }
MPRecoverWait	<- 0

walk	<- Walk()
jump	<- Jump()
kBack	<- KnockBack()
attack	<- Attack()


class Player extends GameObjectBase
{
	/*
	*	コンストラクタ
	*/
	constructor()
	{
		// テクスチャ登録
		this.RegistTexture( "ieni_num03k.png", "Player", 5, 0, 0, 80, 80 )
		
		// 各パラメータ設定
		local obj = GameObject.IObject()
		obj.Texture	= "Player"
		
		// コリジョン設定
		local detail = GameObject.SColDetail.ctorRect( -12, -22, 12, 40 )
		detail.Flag = (ColFlagD.ColMap | ColFlagD.ColObj)
		obj.Col.PushDetail( detail )
		
		// 登録
		GameObjectBase.constructor( obj, "Player", eObjType.Player )
	}
	
	
	/*
	*	初期化
	*/
	function Init(o)
	{
		local c = GameObject.CreateObjectFromName( "Sub01", o.Pos, DIR_INVALID )
		o.Link.Link( c )
		
		this.shotValue = ShotValue()

		walk.Init( o )
		jump.Init( o )
		kBack.Init( o )
		attack.Init( o )
	}
	
	
	/*
	*	通常動作
	*/
	function Move(o)
	{
		local pad = My.CPadInterface.GetPad(0)
		
		
		// 更新
		if( --MPRecoverWait <= 0 ){
			MPRecoverWait = 0
			if( (o.MP+=0.3) > o.MaxMP ) o.MP = o.MaxMP
		}
		
		if( !o.Fatal ){
			// ライフゼロでふぇいたるー
			if( o.HP <= 0 ){ o.HP = 0; o.Fatal = true; }
			else{
				// 自動回復
				local aaa = 1.0 + o.MP * 0.005;
				o.HP += aaa;

				if( o.HP > o.MaxHP ) o.HP = o.MaxHP
			}
		}
		else{
			// フェイタルで喰らったらしぬううううう
			if( o.HP < 0 ) o.Valid = false
		}


		jump.Update()
		kBack.Update()
		attack.Update()
		if( o.ColDir & (DIR_LEFT | DIR_RIGHT) ) o.Speed.X = 0
		o.Speed.Y += 0.5
		
		
		if( kBack.ing ){
			// のっくばっく
			kBack.Action()
			
			// その他はリセット
			walk.Finish()
			jump.Finish()
			attack.Finish()
		}
		else{
			attack.Atk( pad )
			
			if( !jump.ing && !attack.ing ) walk.Action( pad )
			jump.Action( pad )
			attack.Action( pad )
			
			if( attack.ing || jump.ing ){
				walk.Finish()
			}
			if( attack.canJump ){
				jump.Jump( pad )
			}
			
			// その他はリセット
			kBack.Finish()
		}
		
		
		
		// 最大スピード制御
		if     ( o.Speed.X < -maxSpeed.X ) o.Speed.X = -maxSpeed.X
		else if( o.Speed.X >  maxSpeed.X ) o.Speed.X =  maxSpeed.X
		
		if( (-5.0 < o.Speed.X) && (o.Speed.X < 5.0) ) maxSpeed.X = 5.0
		else                                          maxSpeed.X = o.Speed.X * (o.Speed.X < 0 ? -1 : 1)
		/*
		if     ( o.Speed.Y < -maxSpeed.Y ) o.Speed.Y = -maxSpeed.Y
		else if( o.Speed.Y >  maxSpeed.Y ) o.Speed.Y =  maxSpeed.Y
		*/

		
		
		playerParam.oldSpeed.Set( o.Speed.X, o.Speed.Y )
		
		
		if( shotValue != null ) shotValue.Make(o)
	}


	/*
	*	描画
	*/
	function Draw(o)
	{
		local srcPoint = { X = 0, Y = 0 }
		o.Turn.X = (o.Dir & DIR_LEFT)
		
		
		if( kBack.ing ){
			kBack.Draw( srcPoint )
		}
		else{
			if( attack.ing ){
				attack.Draw( srcPoint )
			}
			else if( jump.ing ){
				jump.Draw( srcPoint )
			}
			else{
				walk.Draw( srcPoint )
			}
		}
		
		o.Src.X += o.Src.W * srcPoint.X
		o.Src.Y += o.Src.H * srcPoint.Y
	}
}


Player()




class ShotValue
{
	mode		= 0
	chargeFrame	= 0
	
	
	constructor()
	{
	}
	
	function tmpShot(o, speed)
	{
		GameObject.CreateObjectFromName( "Atk01", o.Pos, o.Dir )
		
		o.Speed.X += speed * (o.Dir & DIR_LEFT ? 1 : -1)
		maxSpeed.X = o.Speed.X * (o.Speed.X < 0 ? -1 : 1)
		MPRecoverWait = 30
	}
	
	function Make(o)
	{
		local pad = My.CPadInterface.GetPad(0)
		
		if( pad.IsPush( PAD_BUTTON_04 ) ) if( ++mode > 1 ) mode = 0
		
		--o.L0
		
		if( pad.IsHold( PAD_BUTTON_03 ) )
			if( o.L0 <= 0 )
				MakeSub[mode](o)
		
		
		if( pad.IsPush( PAD_BUTTON_02 ) ){
			if( o.MP >= 5.0 ){
				o.MP -= 5.0
				this.tmpShot( o, 0.0 )
			}
		}
		else if( pad.IsHold( PAD_BUTTON_02 ) ){
			
			local oldChargeMP = o.ChargeMP
			
			o.ChargeMP += 2.0
			if( o.ChargeMP > o.MP ) o.ChargeMP = o.MP
			
			if( ++chargeFrame % 7 == 0 ){
				local pos = rand() % 64 - 32
				GameObject.CreateObjectFromName( "ChargeEffect01", o.Pos + SPoint2DF.ctor( pos, 40 ), DIR_INVALID )
			}
			
			if(    (oldChargeMP < 100) && (100 <= o.ChargeMP)
				|| (oldChargeMP < 200) && (200 <= o.ChargeMP)
				|| (oldChargeMP < 300) && (300 <= o.ChargeMP) ){
				local c = GameObject.CreateObjectFromName( "ChargeEffect02", o.Pos, DIR_INVALID )
				o.Link.Link( c )
			}
		}
		else if( pad.IsPull( PAD_BUTTON_02 ) ){
			
			if( o.ChargeMP < 100.0 ){
			}
			else if( o.ChargeMP < 200.0 ){
				this.tmpShot( o, 10.0 )
				o.MP -= 100.0
			}
			else if( o.ChargeMP < 300.0 ){
				this.tmpShot( o, 10.0 )
				o.MP -= 200.0
			}
			else{
				this.tmpShot( o, 10.0 )
				o.MP -= 300.0
			}
		}
		
		
		if( pad.IsFree( PAD_BUTTON_02 ) ){
			chargeFrame = 0
			o.ChargeMP	= 0
		}
	}
	
	
	MakeSub = [
		function (o)
		{
		},
		
		function (o)
		{
		},
	]
}












class Atk01 extends GameObjectBase
{
	/*
	*	コンストラクタ
	*/
	constructor()
	{
		// テクスチャ登録
		this.RegistTexture( "sub_01.png", "Atk01", 10, 32*0, 64*1, 32, 64 )
		
		// 各パラメータ設定
		local obj = GameObject.IObject()
		obj.Texture	= "Atk01"
		obj.BlendMode	= AB_ADD
		
		// コリジョン設定
		local detail = GameObject.SColDetail.ctorRect( -8, -8, 8, 8 )
		detail.Flag	= ColFlagD.ColMap
		detail.Target = eObjType.Enemy
		detail.Damage = 200
		obj.Col.PushDetail( detail )

		// 登録
		GameObjectBase.constructor( obj, "Atk01", eObjType.PlayerAtk )
	}
	
	
	/*
	*	初期化
	*/
	function Init(o)
	{
		for(local i=0; i<10; ++i){
			local c = GameObject.CreateObjectFromName( "bbbbb", o.Pos, o.Dir )
			c.Speed = getVector2D_Rand( 20.0, 24.0, 2, 0.25, 0.75, 2 )
			c.Scale2D.Set( 0.5 )
			c.F0 = 0.8
			c.F1 = -0.03
		}
		
		o.Speed.X = 28.0 * (o.Dir == DIR_LEFT ? -1 : 1)
		o.Angle = 0.75 + 0.25 * (o.Dir == DIR_LEFT ? -1 : 1)
	}
	
	
	/*
	*	通常動作
	*/
	function Move(o)
	{
		if( o.ColDir ){ o.Valid = false; return }
		
		{
			local c = GameObject.CreateObjectFromName( "bbbbb", o.Pos, o.Dir )
			c.Scale2D.Set( 2.0 )
			c.F0 = 1.0
			c.F1 = -0.2
		}
		{
			local c = GameObject.CreateObjectFromName( "bbbbb", o.Pos, o.Dir )
			c.Pos.X += 32
			c.Speed = getVector2D_Rand( 4, 4, 1, 0.4, 0.6, 2 )
			c.F0 = 1.0
			c.F1 = -0.04
		}
	}
}

Atk01()


class bbbbb extends GameObjectBase
{
	/*
	*	コンストラクタ
	*/
	constructor()
	{
		// テクスチャ登録
		this.RegistTexture( "32.png", "bbbbb", 10, 32*3, 32*7, 32, 32 )
		
		// 各パラメータ設定
		local obj = GameObject.IObject()
		obj.Texture		= "bbbbb"
		obj.BlendMode	= AB_ADD
		/*
		obj.BlendMode	= AB_BLEND
		obj.Color.a = 0.1
		*/

		// 登録
		GameObjectBase.constructor( obj, "bbbbb", eObjType.Effect )
	}
	
	
	/*
	*	通常動作
	*/
	function Move(o)
	{
		o.Speed.Mul( o.F0 )
		o.Scale2D.Add( o.F1 )
		
		if( o.Scale2D.X < 0 ) o.Valid = false
	}
}

bbbbb()
