﻿/*
*	じゃんぷ
*/


enum eJumpState
{
	Ready,		// デフォルト。ジャンプ前の硬直時間
	Rise,		// 上昇中
	Top,		// 頂点付近
	Fall,		// 落下中
	Land1,		// 着地硬直（弱）
	Land2,		// 着地硬直（強）
	WallReadyR,	// 壁けり前の硬直時間（上り）
	WallReadyF,	// 壁けり前の硬直時間（下り）
	WallRise,	// 壁けり上昇中。上昇時間が短めなので分けた
	Brake,		// 高速ブレーキ
}


class Jump
{
	obj		= null
	
	ing		= false
	state	= eJumpState.Ready
	frame	= -1
	dir		= DIR_INVALID
	
	
	/*
	*	
	*/
	function Init(o)
	{
		obj	= o
	}
	
	
	/*
	*	あっぷでーと
	*/
	function Update()
	{
		// 空中なら強制ジャンプ状態
		if( !(obj.ColDir & DIR_DOWN) ) ing = true
	}
	
	
	/*
	*	あくしょん
	*/
	function Action(pad)
	{
		if( ing ){
			++frame
			local oldState = state
			
			
			// ずつき
			switch( state ){
				case eJumpState.Rise:
				case eJumpState.WallRise:
				case eJumpState.Top:
					if( obj.ColDir & DIR_UP ){
						state	= eJumpState.Fall
						frame	= 0
					}
					break
			}
			
			// 着地
			switch( state ){
				case eJumpState.Ready:
				case eJumpState.Land1:
				case eJumpState.Land2:
				case eJumpState.WallReadyR:
				case eJumpState.WallReadyF:
				case eJumpState.Brake:
					break
					
				default:
					if( obj.ColDir & DIR_DOWN ){
						local tmpSpeed = obj.Speed.X * (obj.Speed.X < 0 ? -1 : 1)
						if( tmpSpeed > 8 ){
							state	= eJumpState.Brake
						}
						else{
							obj.Speed.X = 0
							state	= (playerParam.oldSpeed.Y < 15.0 ? eJumpState.Land1 : eJumpState.Land2)
						}
						
						frame	= 0
					}
					break
			}
			
			
			// 空中操作
			switch( state ){
				case eJumpState.Rise:
				case eJumpState.Top:
				case eJumpState.Fall:
				case eJumpState.WallRise:
				
					local hold = {
						left	= pad.IsHold( PAD_DIR_LEFT ),
						right	= pad.IsFree( PAD_DIR_LEFT ) && pad.IsHold( PAD_DIR_RIGHT ),
					}
					
					// 左右入力
					obj.Speed.X += 0.2 * (hold.left ? -1 : (hold.right ? 1 : 0))
					
					// かべけり
					if( pad.IsPush( PAD_BUTTON_01 ) ){
						local left	= (obj.Dir & DIR_LEFT)  && (obj.ColDir & DIR_LEFT)  && hold.left
						local right	= (obj.Dir & DIR_RIGHT) && (obj.ColDir & DIR_RIGHT) && hold.right
						
						if( left || right ){
							state	= (obj.Speed.Y < 0 ? eJumpState.WallReadyR : eJumpState.WallReadyF)
							frame	= 0
							dir		= (left ? DIR_RIGHT : DIR_LEFT)
							
							obj.Speed.Set( 0 )
						}
					}
					break
			}
			
			
			// 各状態
			switch( state ){
				case eJumpState.Ready:
					// ため
					if( frame == 0 ){
						state = eJumpState.Fall
					}
					else if( frame >= 4 ){
						state		= eJumpState.Rise
						frame		= 0
						obj.Speed	= this.GetJumpSpeed()
					}
					break
					
				case eJumpState.Rise:
				case eJumpState.WallRise:
					// あがり
					local limitFrame = (state == eJumpState.Rise ? 11 : 6)
					
					if( frame < limitFrame ){
						obj.Speed.Y = this.GetJumpSpeed().Y
						if( pad.IsFree( PAD_BUTTON_01 ) ) frame = limitFrame
					}
					else{
						obj.Speed.Y += 0.8
						if( obj.Speed.Y > -4.0 ){
							state	= eJumpState.Top
							frame	= 0
						}
					}
					break
					
				case eJumpState.Top:
					// ちょーてん
					if( obj.Speed.Y > 0 ){
						state	= eJumpState.Fall
						frame	= 0
					}
					break
					
				case eJumpState.Fall:
					// らっか
					break
					
				case eJumpState.Land1:
					// ちゃくち
					if( frame >= 2 ) this.Finish()
					this.Jump( pad )
				
					if( pad.IsPush( PAD_BUTTON_01 ) ) obj.Dir = (dir != DIR_INVALID ? dir : obj.Dir)
					break
					
				case eJumpState.Land2:
					// ちゃくち（大）
					if( frame >= 16 ) this.Finish()
					break;
					
				case eJumpState.WallReadyR:
				case eJumpState.WallReadyF:
					// 壁けり始動
					obj.Speed.Y = 0
					
					if( frame >= 4 ){
						state		= eJumpState.WallRise
						frame		= 0
						obj.Dir		= dir
						obj.Speed	= this.GetJumpSpeed()
					}
					break
					
				case eJumpState.Brake:
					if( obj.ColDir & DIR_DOWN ){
						local downSpeed = 0.8
						
						if( obj.Speed.X < 0 ){
							obj.Speed.X += downSpeed
							if( obj.Speed.X > 0 ) obj.Speed.X = 0
						}
						else if( obj.Speed.X > 0 ){
							obj.Speed.X -= downSpeed
							if( obj.Speed.X < 0 ) obj.Speed.X = 0
						}
						
						
						if( obj.Speed.X != 0 ){
							frame = 0
						}
						else{
							if     ( frame == 8 ) obj.Pos.X += 8 * (obj.Dir & DIR_LEFT ? 1 : -1)
							else if( frame > 12 ) this.Finish()
						}
					}
					else{
						state	= eJumpState.Fall
						frame	= 0
					}
					break
			}
		}
	}
	
	
	/*
	*	地上にいるときのジャンプ
	*/
	function Jump(pad)
	{
		if( !ing && pad.IsPush( PAD_BUTTON_01 ) ){
			obj.Speed.X = 0
			
			ing		= true
			state	= eJumpState.Ready
			frame	= 0
			dir		= (pad.IsHold( PAD_DIR_LEFT ) ? DIR_LEFT : (pad.IsHold( PAD_DIR_RIGHT ) ? DIR_RIGHT : DIR_INVALID))
		}
	}
	
	
	/*
	*	ジャンプ時のスピード
	*/
	function GetJumpSpeed()
	{
		return SPoint2DF.ctor( 5.0 * (dir & DIR_LEFT ? -1 : (dir & DIR_RIGHT ? 1 : 0)), -9 )
	}
	
	
	/*
	*	
	*/
	function Finish()
	{
		ing		= false
		state	= eJumpState.Ready
		frame	= -1
	}
	
	
	/*
	*	描画
	*/
	function Draw(srcPoint)
	{
		switch( state ){
			case eJumpState.Ready:
				// ため
				srcPoint.Y = 3
				
				local src = [ 0, 1, null ]
				
				foreach(i, v in src)
					if( frame < 2 * (i+1) ){
						if( v == null ){ srcPoint.X = src[0]; frame = 0 }
						else             srcPoint.X = v
						break
					}
				break
				
			case eJumpState.Rise:
			case eJumpState.WallRise:
				// あがり
				srcPoint.Y = 3
				
				local src = [ 2, 3 ]
				local tmp = frame % 10
				
				foreach(i, v in src)
					if( tmp < 5 * (i+1) ){ srcPoint.X = v; break }
				break
				
			case eJumpState.Top:
				// ちょーてん
				srcPoint.Y = 3
				
				if( frame < 5 ) srcPoint.X = 4
				else            srcPoint.X = 5
				break
				
			case eJumpState.Fall:
				// らっか
				srcPoint.Y = 4
				
				local src = [ 0, 1, 2, null ]
				
				foreach(i, v in src)
					if( frame < 5 * (i+1) ){
						if( v == null ){ srcPoint.X = src[1]; frame = 5 }
						else             srcPoint.X = v
						break
					}
				break
				
			case eJumpState.Land1:
				// ちゃくち
				srcPoint.X = 5
				
				if( frame < 2 ) srcPoint.Y = 5
				else            srcPoint.Y = 4
				break
				
			case eJumpState.Land2:
				// ちゃくち（大）
				srcPoint.Y = 4
				
				local src = [ 3, 4, 5, null ]
				
				if     ( frame < 6 ) srcPoint.X = src[0]
				else if( frame < 12 ) srcPoint.X = src[1]
				else                 srcPoint.X = src[2]
				break
				
			case eJumpState.WallReadyR:
				srcPoint.Y = 5
				
				if( frame < 2 ) srcPoint.X = 2
				else            srcPoint.X = 3
				break
				
			case eJumpState.WallReadyF:
				srcPoint.Y = 5
				
				if( frame < 2 ) srcPoint.X = 0
				else            srcPoint.X = 1
				break
				
			case eJumpState.Brake:
				srcPoint.Y = 2
				
				if     ( frame < 0 ) srcPoint.X = 3
				else if( frame < 8 ) srcPoint.X = 4
				else                 srcPoint.X = 5
				break
		}
	}
}
