﻿/*
*	プレイヤー
*/

class SheepParam extends CSqParam
{
}


enum eSheepState
{
	Invalid,
	Stand,
	Walk,
	Dush,
	Fall,
	DushFall,
	Jump,
	DushJump,
	BackStep,
	Dead,
}

class Sheep
{
	state		= eSheepState.Stand
	stateSub	= 0
	frame		= 0
	frameShadow	= 0
	bound		= false
}

sheep <- Sheep()



class PlayerSheep extends GameObjectBase
{
	/*
	*	コンストラクタ
	*/
	constructor()
	{
		// テクスチャ登録
		this.RegistTexture( "sheep.png", "PlayerSheep", 50001, 0, 0, 80, 80 )
		
		// 各パラメータ設定
		local obj = GameObject.IObject()
		obj.Texture	= "PlayerSheep"
		obj.ExtendAliveArea.Sy	= -640
		
		// コリジョン設定
		local detail = GameObject.SColDetail.ctorRect( -12, -32, 12, 40 )
		detail.Flag = (ColFlagD.ColMap | ColFlagD.ColObj)
		obj.Col.PushDetail( detail )
		
		// 登録
		GameObjectBase.constructor( obj, "PlayerSheep", eObjType.Player )
	}
	
	
	/*
	*	初期化
	*/
	function Init(o)
	{
		o.SqParam	= SheepParam()
	}
	
	
	/*
	*	通常動作
	*/
	function Move(o)
	{
		local prm = o.SqParam
		prm.Update()
		
		local pad = My.CPadInterface.GetPad(0)
		
		local input = {
			down		= pad.IsHold( PAD_DIR_DOWN ),
			left		= pad.IsHold( PAD_DIR_LEFT ),
			right		= pad.IsHold( PAD_DIR_RIGHT ) && pad.IsFree( PAD_DIR_LEFT ),
			dush		= pad.IsHold( PAD_DIR_LEFT ) && pad.IsHold( PAD_BUTTON_01 ),
			jump		= pad.IsFree( PAD_DIR_DOWN ) && pad.IsPush( PAD_BUTTON_02 ),
			backStep	= pad.IsHold( PAD_DIR_DOWN ) && pad.IsPush( PAD_BUTTON_02 ),
		}
		
		
		// ぶつかったら減速（ほぼ停止ですけど）
		if( ((o.Speed.Y < 0) && (o.ColDir & DIR_UP)) || ((o.Speed.Y > 0) && (o.ColDir & DIR_DOWN)) ) o.Speed.Y = 0.0
		//else if( prm.boundDir & (DIR_LEFT | DIR_RIGHT) ) o.Speed.X *= 0.5
		
		//if( ((o.Speed.X < 0) && (o.ColDir & DIR_LEFT)) || ((o.Speed.X > 0) && (o.ColDir & DIR_RIGHT)) ) o.Speed.X *= 0.5
		if( ((o.Speed.X < 0) && (o.ColDir & DIR_LEFT)) || ((o.Speed.X > 0) && (o.ColDir & DIR_RIGHT)) ) o.Speed.X = 0
		else if( prm.boundDir & (DIR_LEFT | DIR_RIGHT) ) o.Speed.X *= 0.5
		
		// 旧すてーと
		local stateOld = sheep.state.tointeger()
		
		// 落下速度
		o.Speed.Y += 0.45
		
		++sheep.frame
		
		
		// 接触リセット？
		if( prm.boundDir ){
			switch( prm.boundDir ){
				case DIR_UP:
					break
					
				case DIR_DOWN:
					switch( sheep.state ){
						case eSheepState.Stand:
							break
						case eSheepState.Walk:
						case eSheepState.Jump:
							sheep.state = eSheepState.Fall
							break
						case eSheepState.Dush:
						case eSheepState.DushFall:
						case eSheepState.DushJump:
							sheep.state = (input.dush ? eSheepState.DushFall : eSheepState.Fall)
							break
						case eSheepState.BackStep:
							sheep.state = eSheepState.Fall
							break
					}
					if( pad.IsHold( PAD_BUTTON_02 ) ) o.Speed.Y *= 1.4
					sheep.frame	= 0
					break
					
				case DIR_LEFT:
				case DIR_RIGHT:
					switch( sheep.state ){
						case eSheepState.BackStep:
							sheep.state = eSheepState.Stand
							break
					}
					break
			}
			prm.boundDir = DIR_INVALID
			sheep.bound = true
		}
		else{
			sheep.bound = false
		}
		
		if( o.HP <= 0 ) sheep.state = eSheepState.Dead
		
		
		
		switch( sheep.state ){
			case eSheepState.Stand:
				if     ( input.backStep         ) sheep.state = eSheepState.BackStep
				else if( !(o.ColDir & DIR_DOWN) ) sheep.state = (input.dush ? eSheepState.DushFall : eSheepState.Fall)
				else if( input.jump             ) sheep.state = (input.dush ? eSheepState.DushJump : eSheepState.Jump)
				else if( input.dush             ) sheep.state = eSheepState.Dush
				else if( input.left             ) sheep.state = eSheepState.Walk
				break
				
			case eSheepState.Walk:
				if( !input.left && (o.Speed.X == 0.0) ) sheep.state = eSheepState.Stand
				
				if     ( input.backStep         ) sheep.state = eSheepState.BackStep
				else if( !(o.ColDir & DIR_DOWN) ) sheep.state = (input.dush ? eSheepState.DushFall : eSheepState.Fall)
				else if( input.jump             ) sheep.state = (input.dush ? eSheepState.DushJump : eSheepState.Jump)
				else if( input.dush             ) sheep.state = eSheepState.Dush
				break
				
			case eSheepState.Dush:
				if( !input.dush && (o.Speed.X >= -4.0) ) sheep.state = eSheepState.Walk
				
				if     ( input.backStep         ) sheep.state = eSheepState.BackStep
				else if( !(o.ColDir & DIR_DOWN) ) sheep.state = (input.dush ? eSheepState.DushFall : eSheepState.Fall)
				else if( input.jump             ) sheep.state = (input.dush ? eSheepState.DushJump : eSheepState.Jump)
				break
				
			case eSheepState.Fall:
			case eSheepState.DushFall:
				if( o.ColDir & DIR_DOWN ){
					if     ( o.Speed.X == 0.0 ) sheep.state = eSheepState.Stand
					else if( o.Speed.Y >= 0.0 ) sheep.state = (input.dush ? eSheepState.Dush : eSheepState.Walk)
				}
				break
				
			case eSheepState.Jump:
			case eSheepState.DushJump:
				if( o.ColDir & DIR_DOWN ){
					if     ( o.Speed.X == 0.0 ) sheep.state = eSheepState.Stand
					else if( o.Speed.Y >= 0.0 ) sheep.state = (input.dush ? eSheepState.Dush : eSheepState.Walk)
				}
				break
				
			case eSheepState.BackStep:
				local oldStateSub = sheep.stateSub
				
				switch( sheep.stateSub ){
					case 0:
						if( sheep.frame >= 4 ) ++sheep.stateSub
						break
					case 1:
						//print( o.Speed.Y + "\n" )
						
						if( (o.ColDir & DIR_DOWN) ){
							++sheep.stateSub; sheep.frame = 0
							if( !sheep.bound ) o.Speed.X = 0
							local xau = My.CXAudio()
							local se = xau.Find("back_step2")
							se.Play()
						}
						break
					case 2:
						if     ( input.backStep   ){ sheep.stateSub = 0; stateOld = eSheepState.Invalid }
						else if( input.jump       ) sheep.state = eSheepState.Jump
						else if( sheep.frame >= 4 ) sheep.state = eSheepState.Stand
						
						if( (sheep.stateSub != 2) || (sheep.state != eSheepState.BackStep) ){
							sheep.stateSub	= 0
							sheep.frame		= 0
						}
						break
				}
				break
				
			case eSheepState.Dead:
				break
		}
		
		
		// 切り替えリセット
		local notChangeReset = ((stateOld == eSheepState.Walk) && (sheep.state == eSheepState.Dush))
							|| ((stateOld == eSheepState.Dush) && (sheep.state == eSheepState.Walk))
		
		if( sheep.state != stateOld ){
			if( !notChangeReset ){
				sheep.stateSub	= 0
				sheep.frame		= 0
				
				switch( sheep.state ){
					case eSheepState.BackStep:
						o.Speed.Set( 6 - 0.6 * 0.8, -5 )	
						break
						
					case eSheepState.Dead:
						o.Speed.Set( 0, -6 )
						o.Col.Clear()
						break
				}
			}
			
			
			// 音
			local xau = My.CXAudio()
			local se = null
			
			switch( sheep.state ){
				case eSheepState.Dush:
					se = xau.Find("dush")
					break
				case eSheepState.BackStep:
					se = xau.Find("back_step")
					break
				case eSheepState.Jump:
				case eSheepState.DushJump:
					se = xau.Find("jump")
					break
			}
			
			if( se != null ) se.Play()
		}
		if( sheep.state != eSheepState.Dush ){
			local xau = My.CXAudio()
			local se = xau.Find("dush")
			if( se != null ) se.Stop()
		}
		
		
		
		// 横移動
		switch( sheep.state ){
			case eSheepState.Stand:
			case eSheepState.Walk:
			case eSheepState.Dush:
			case eSheepState.Fall:
			case eSheepState.DushFall:
			case eSheepState.Jump:
			case eSheepState.DushJump:
			
				local isDush = (sheep.state == eSheepState.Dush)
				
				local maxSpd = o.Speed.X * (o.Speed.X < 0 ? -1 : 1)
				
				if( isDush && (o.ColDir & DIR_DOWN)     ) maxSpd = 10
				if( (-4 < o.Speed.X) && (o.Speed.X < 4) ) maxSpd = 4
				
				
				local axlSpeed = 0.0
				
				if     ( input.left                                                 ) axlSpeed = -1.0
				else if( input.right && (!(o.ColDir & DIR_DOWN) || (o.Speed.X < 0)) ) axlSpeed =  1.0
				else                                                                  axlSpeed =  0.8 * (o.Speed.X < 0 ? 1.0 : (o.Speed.X > 0 ? -1.0 : 0.0))
				
				if( !input.dush && (maxSpd > 4) ){
					axlSpeed = 0.8 * (o.Speed.X < 0 ? 1.0 : (o.Speed.X > 0 ? -1.0 : 0.0))
				}
				
				local oldSpeed = o.Speed.X
				o.Speed.X += axlSpeed * (o.ColDir & DIR_DOWN ? 0.6 : 0.2)
				
				// 放置停止
				if( ((oldSpeed < 0) && (o.Speed.X > 0)) || ((oldSpeed > 0) && (o.Speed.X < 0)) ) o.Speed.X = 0
				
				if     ( o.Speed.X < -maxSpd ) o.Speed.X = -maxSpd
				else if( o.Speed.X >  maxSpd ) o.Speed.X =  maxSpd
				
				break
		}
		
		
		// ジャンプ
		switch( sheep.state ){
			case eSheepState.Jump:
			case eSheepState.DushJump:
				if( sheep.frame < 17 ){
					if( pad.IsHold( PAD_BUTTON_02 ) && !(o.ColDir & DIR_UP) ) o.Speed.Y   = -8.3
					else                                                      sheep.frame = 17
				}
				if( o.Speed.Y < -2.0 ) o.Speed.Y += 1.2
				break
		}
	}


	/*
	*	描画
	*/
	function Draw(o)
	{
		local srcPoint = { X = 0, Y = 0 }
		//o.Turn.X = (o.Dir & DIR_LEFT)
		o.ApplyDefSrc()
		
		
		switch( sheep.state ){
			case eSheepState.Stand:
				break
				
			case eSheepState.Walk:
			case eSheepState.Dush:
				if( o.Speed.X != 0.0 ){
					
					sheep.frame += 10 * (-o.Speed.X / 20.0) - 1
					
					local totalFrame = 10 * 3
					if     ( sheep.frame <           0 ) sheep.frame += totalFrame
					else if( sheep.frame >= totalFrame ) sheep.frame -= totalFrame

					foreach(i, v in [ 0, 1, 2 ])
						if( sheep.frame < 10 * (i+1) ){ srcPoint.X = v; break }
					srcPoint.Y = 1
				}
				break
				
			case eSheepState.Fall:
			case eSheepState.DushFall:
			case eSheepState.Jump:
			case eSheepState.DushJump:
				srcPoint.Y = 2
				break
				
			case eSheepState.BackStep:
				srcPoint.X = [ 2, 1, 2 ][ sheep.stateSub ]
				break
				
			case eSheepState.Dead:
				break
		}


		o.Src.X += o.Src.W * srcPoint.X
		o.Src.Y += o.Src.H * srcPoint.Y
		
		
			
		// 残像
		switch( sheep.state ){
			case eSheepState.Stand:
			case eSheepState.Walk:
			case eSheepState.Fall:
			case eSheepState.Jump:
			case eSheepState.Dead:
				sheep.frameShadow	= 0
				break
				
			case eSheepState.Dush:
			case eSheepState.DushFall:
			case eSheepState.DushJump:
			case eSheepState.BackStep:
				
				if( --sheep.frameShadow <= 0 ){
					sheep.frameShadow = 2
					PlayerShadow.Make( o.OldPos, o.Src.Clone() )
				}
				break
		}
	}
}


PlayerSheep()
